@@ -18,6 +18,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <glib.h>
 #include <glib-object.h>
 
 #include <bonobo/bonobo-widget.h>
@@ -1217,8 +1218,7 @@ bonobo_ui_engine_widget_set (BonoboUIEngine    *engine,
 
 	if (custom_widget) {
 		info = bonobo_ui_xml_get_data (engine->priv->tree, node);
-		info->widget = gtk_widget_ref (custom_widget);
-		gtk_object_sink (GTK_OBJECT (custom_widget));
+		info->widget = g_object_ref_sink (custom_widget);
 
 		bonobo_ui_engine_stamp_custom (engine, node);
 	}
@@ -2372,7 +2372,7 @@ check_excess_widgets (BonoboUISync *sync, GList *wptr)
 
 			node = bonobo_ui_engine_widget_get_node (b->data);
 			g_message ("Widget type '%s' with node: '%s'",
-				   G_OBJECT_CLASS_NAME (b->data) ? G_OBJECT_CLASS_NAME (b->data) : "NULL",
+				   G_OBJECT_CLASS_NAME (G_OBJECT_GET_CLASS (b->data)) ? G_OBJECT_CLASS_NAME (G_OBJECT_GET_CLASS (b->data)) : "NULL",
 				   node ? bonobo_ui_xml_make_path (node) : "NULL");
 		}
 	}
@@ -3158,7 +3158,7 @@ bonobo_ui_engine_thaw (BonoboUIEngine *engine)
  * 
  * This is a debugging function mostly for internal
  * and testing use, it dumps the XML tree, including
- * the assoicated, and overridden nodes in a wierd
+ * the associated, and overridden nodes in a wierd
  * hackish format to the @out stream with the
  * helpful @msg prepended.
  **/
