@@ -282,6 +282,32 @@ label_same (GtkBin *menu_widget, const char *txt)
 		!strcmp (((GtkLabel *)label)->label, txt);
 }
 
+static gboolean
+widget_has_accel (GtkWidget       *widget,
+		  GtkAccelGroup   *accel_group,
+		  guint           key, 
+		  GdkModifierType mods)
+{
+	int i;
+	guint n_entries;
+	GList *l_tmp, *l_closures = gtk_widget_list_accel_closures (widget);
+	GtkAccelGroupEntry *entry = gtk_accel_group_query (accel_group,
+							   key, mods,
+							   &n_entries);
+
+	if (n_entries)
+		for (l_tmp = l_closures; l_tmp; l_tmp = l_tmp->next)
+	  		for (i=0; i<n_entries; i++) 
+			  	if (entry[i].closure == l_tmp->data) {
+					g_list_free (l_closures);
+					return TRUE;
+				}
+	
+	g_list_free (l_closures);
+
+	return FALSE;
+}
+
 static void
 impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 				BonoboUINode *node,
@@ -392,11 +418,12 @@ impl_bonobo_ui_sync_menu_state (BonoboUISync *sync,
 		if (!key) /* FIXME: this looks strange */
 			return;
 
-		gtk_widget_add_accelerator (menu_widget,
-					    "activate",
-					    sync_menu->accel_group,
-					    key, mods,
-					    GTK_ACCEL_VISIBLE);
+	  	if (! widget_has_accel (menu_widget, sync_menu->accel_group, key, mods))
+			gtk_widget_add_accelerator (menu_widget,
+						    "activate",
+						    sync_menu->accel_group,
+						    key, mods,
+						    GTK_ACCEL_VISIBLE);
 	}
 
 	bonobo_ui_engine_queue_update (
@@ -757,12 +784,21 @@ impl_bonobo_ui_sync_menu_update_root (BonoboUISync *sync,
 				      BonoboUINode *root)
 {
 	BonoboUISyncMenu *smenu = BONOBO_UI_SYNC_MENU (sync);
+	const char *txt;
+	gboolean    detachable;
 
 	if (bonobo_ui_node_has_name (root, "menu") &&
 	    smenu->menu_dock_item)
 		bonobo_ui_sync_do_show_hide (
 			sync, root, NULL,
 			smenu->menu_dock_item);
+
+	if ((txt = bonobo_ui_node_peek_attr (root, "behavior")) &&
+	    strstr (txt, "detachable"))
+		detachable = TRUE;
+	else
+	    detachable = bonobo_ui_preferences_get_menubar_detachable ();
+	bonobo_dock_item_set_locked (smenu->menu_dock_item, !detachable);
 }
 
 static void
