@@ -39,7 +39,13 @@ add_control (GtkBox *box)
 {
 	GtkWidget *tmp, *control;
 
+#if 1
 	control = bonobo_widget_new_control ("OAFIID:Bonobo_Sample_Entry", NULL);
+#else
+	control = bonobo_widget_new_control ("OAFIID:Nautilus_Text_View", NULL);
+#endif
+
+	g_assert (control != NULL);
 	gtk_box_pack_start_defaults (box, control);
 	gtk_widget_show (control);
 
@@ -63,7 +69,6 @@ main (int argc, char **argv)
 	GtkWidget *tmp;
 	GtkWidget *window;
 	GtkWidget *vbox;
-	CORBA_ORB  orb;
 
 	free (malloc (8));
 
@@ -73,8 +78,6 @@ main (int argc, char **argv)
 			    LIBBONOBOUI_MODULE,
 			    argc, argv, NULL);
 
-	orb = bonobo_orb ();
-
 	bonobo_activate ();
 
 	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
